@@ -27,6 +27,7 @@ import java.util.concurrent.CountDownLatch;
 import example.domain.Person;
 import example.util.Calculator;
 
+import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.api.Test;
 
 class AssertionsDemo {
@@ -91,6 +92,9 @@ class AssertionsDemo {
 		assertEquals("/ by zero", exception.getMessage());
 	}
 
+	// end::user_guide[]
+	@Tag("timeout")
+	// tag::user_guide[]
 	@Test
 	void timeoutNotExceeded() {
 		// The following assertion succeeds.
@@ -99,6 +103,9 @@ class AssertionsDemo {
 		});
 	}
 
+	// end::user_guide[]
+	@Tag("timeout")
+	// tag::user_guide[]
 	@Test
 	void timeoutNotExceededWithResult() {
 		// The following assertion succeeds, and returns the supplied object.
@@ -108,6 +115,9 @@ class AssertionsDemo {
 		assertEquals("a result", actualResult);
 	}
 
+	// end::user_guide[]
+	@Tag("timeout")
+	// tag::user_guide[]
 	@Test
 	void timeoutNotExceededWithMethod() {
 		// The following assertion invokes a method reference and returns an object.
@@ -116,6 +126,7 @@ class AssertionsDemo {
 	}
 
 	// end::user_guide[]
+	@Tag("timeout")
 	@extensions.ExpectToFail
 	// tag::user_guide[]
 	@Test
@@ -129,6 +140,7 @@ class AssertionsDemo {
 	}
 
 	// end::user_guide[]
+	@Tag("timeout")
 	@extensions.ExpectToFail
 	// tag::user_guide[]
 	@Test
