@@ -1,25 +1,30 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package example;
 
 // @formatter:off
 // tag::user_guide[]
+
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assumptions.assumeTrue;
 import static org.junit.jupiter.api.Assumptions.assumingThat;
 
+import example.util.Calculator;
+
 import org.junit.jupiter.api.Test;
 
 class AssumptionsDemo {
 
+	private final Calculator calculator = new Calculator();
+
 	@Test
 	void testOnlyOnCiServer() {
 		assumeTrue("CI".equals(System.getenv("ENV")));
@@ -38,11 +43,11 @@ class AssumptionsDemo {
 		assumingThat("CI".equals(System.getenv("ENV")),
 			() -> {
 				// perform these assertions only on the CI server
-				assertEquals(2, 2);
+				assertEquals(2, calculator.divide(4, 2));
 			});
 
 		// perform these assertions in all environments
-		assertEquals("a string", "a string");
+		assertEquals(42, calculator.multiply(6, 7));
 	}
 
 }
