@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package example;
@@ -15,7 +15,8 @@ import org.junit.Test;
 public class JUnit4Tests {
 
 	@Test
-	public void standardJUnit4Test() throws Exception {
-		/* no-op */
+	public void standardJUnit4Test() {
+		// perform assertions
 	}
+
 }
