@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package example.defaultmethods;
@@ -15,17 +15,17 @@ class StringTests implements ComparableContract<String>, EqualsContract<String>
 
 	@Override
 	public String createValue() {
-		return "foo";
+		return "banana";
 	}
 
 	@Override
 	public String createSmallerValue() {
-		return "bar"; // 'b' < 'f' in "foo"
+		return "apple"; // 'a' < 'b' in "banana"
 	}
 
 	@Override
 	public String createNotEqualValue() {
-		return "baz";
+		return "cherry";
 	}
 
 }
