@@ -1,16 +1,15 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 package example.registration
 
 import org.junit.jupiter.api.Assertions.assertEquals
-
 import org.junit.jupiter.api.Test
 import org.junit.jupiter.api.extension.RegisterExtension
 
@@ -18,11 +17,11 @@ import org.junit.jupiter.api.extension.RegisterExtension
 class KotlinWebServerDemo {
 
     companion object {
-        @JvmField
+        @JvmStatic
         @RegisterExtension
         val server = WebServerExtension.builder()
-                .enableSecurity(false)
-                .build()
+            .enableSecurity(false)
+            .build()
     }
 
     @Test
