@@ -10,7 +10,7 @@
 
 package org.junit.jupiter.api.parallel;
 
-import static org.apiguardian.api.API.Status.EXPERIMENTAL;
+import static org.apiguardian.api.API.Status.STABLE;
 
 import java.lang.annotation.ElementType;
 import java.lang.annotation.Inherited;
@@ -33,7 +33,7 @@ import org.apiguardian.api.API;
  * @see ExecutionMode
  * @see ResourceLock
  */
-@API(status = EXPERIMENTAL, since = "5.7")
+@API(status = STABLE, since = "5.10")
 @Retention(RetentionPolicy.RUNTIME)
 @Target(ElementType.TYPE)
 @Inherited
