@@ -1,26 +1,25 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.jupiter;
 
-import org.junit.platform.runner.JUnitPlatform;
 import org.junit.platform.suite.api.IncludeClassNamePatterns;
 import org.junit.platform.suite.api.IncludeEngines;
 import org.junit.platform.suite.api.SelectPackages;
-import org.junit.runner.RunWith;
+import org.junit.platform.suite.api.Suite;
 
 /**
  * Test suite for the JUnit Jupiter programming model, extension model, and
  * {@code TestEngine} implementation.
  *
- * <h3>Logging Configuration</h3>
+ * <h2>Logging Configuration</h2>
  *
  * <p>In order for our log4j2 configuration to be used in an IDE, you must
  * set the following system property before running any tests &mdash; for
@@ -32,9 +31,9 @@ import org.junit.runner.RunWith;
  *
  * @since 5.0
  */
-@RunWith(JUnitPlatform.class)
+@Suite
 @SelectPackages("org.junit.jupiter")
 @IncludeClassNamePatterns(".*Tests?")
 @IncludeEngines("junit-jupiter")
-public class JupiterTestSuite {
+class JupiterTestSuite {
 }
