@@ -12,6 +12,7 @@ package org.junit.jupiter.api;
 
 import static org.junit.jupiter.api.AssertionTestUtils.assertExpectedAndActualValues;
 import static org.junit.jupiter.api.AssertionTestUtils.assertMessageContains;
+import static org.junit.jupiter.api.AssertionTestUtils.assertMessageMatches;
 import static org.junit.jupiter.api.AssertionTestUtils.assertMessageStartsWith;
 import static org.junit.jupiter.api.AssertionTestUtils.expectAssertionFailedError;
 import static org.junit.jupiter.api.Assertions.assertSame;
@@ -82,6 +83,20 @@ class AssertSameAssertionsTests {
 		}
 	}
 
+	@Test
+	void assertSameWithEqualPrimitivesAutoboxedToDifferentWrappers() {
+		try {
+			int i = 999;
+			assertSame(i, i);
+			expectAssertionFailedError();
+		}
+		catch (AssertionFailedError ex) {
+			assertMessageMatches(ex,
+				"expected: java\\.lang\\.Integer@.+?<999> but was: java\\.lang\\.Integer@.+?<999>");
+			assertExpectedAndActualValues(ex, 999, 999);
+		}
+	}
+
 	@Test
 	void assertSameWithEquivalentStringsAndMessageSupplier() {
 		String expected = new String("foo");
