@@ -24,6 +24,7 @@ import static org.junit.jupiter.api.condition.EnabledOnJreIntegrationTests.onJav
 import static org.junit.jupiter.api.condition.EnabledOnJreIntegrationTests.onJava19;
 import static org.junit.jupiter.api.condition.EnabledOnJreIntegrationTests.onJava20;
 import static org.junit.jupiter.api.condition.EnabledOnJreIntegrationTests.onJava21;
+import static org.junit.jupiter.api.condition.EnabledOnJreIntegrationTests.onJava22;
 import static org.junit.jupiter.api.condition.EnabledOnJreIntegrationTests.onJava8;
 import static org.junit.jupiter.api.condition.EnabledOnJreIntegrationTests.onJava9;
 
@@ -205,6 +206,15 @@ class EnabledOnJreConditionTests extends AbstractExecutionConditionTests {
 		assertEnabledOnCurrentJreIf(onJava21());
 	}
 
+	/**
+	 * @see EnabledOnJreIntegrationTests#java22()
+	 */
+	@Test
+	void java22() {
+		evaluateCondition();
+		assertEnabledOnCurrentJreIf(onJava22());
+	}
+
 	/**
 	 * @see EnabledOnJreIntegrationTests#other()
 	 */
@@ -213,7 +223,7 @@ class EnabledOnJreConditionTests extends AbstractExecutionConditionTests {
 		evaluateCondition();
 		assertEnabledOnCurrentJreIf(
 			!(onJava8() || onJava9() || onJava10() || onJava11() || onJava12() || onJava13() || onJava14() || onJava15()
-					|| onJava16() || onJava17() || onJava18() || onJava19() || onJava20() || onJava21()));
+					|| onJava16() || onJava17() || onJava18() || onJava19() || onJava20() || onJava21() || onJava22()));
 		assertCustomDisabledReasonIs("Disabled on almost every JRE");
 	}
 
