@@ -30,6 +30,7 @@ import java.io.File;
 import java.io.IOException;
 import java.nio.file.Files;
 import java.nio.file.Path;
+import java.nio.file.attribute.DosFileAttributeView;
 import java.util.Deque;
 import java.util.LinkedList;
 import java.util.function.Supplier;
@@ -47,13 +48,16 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.TestInfo;
 import org.junit.jupiter.api.TestInstance;
 import org.junit.jupiter.api.TestMethodOrder;
-import org.junit.jupiter.api.condition.DisabledOnOs;
-import org.junit.jupiter.api.condition.OS;
+import org.junit.jupiter.api.extension.AnnotatedElementContext;
 import org.junit.jupiter.api.extension.ExtensionConfigurationException;
+import org.junit.jupiter.api.extension.ExtensionContext;
 import org.junit.jupiter.api.extension.ParameterResolutionException;
 import org.junit.jupiter.api.io.TempDir;
+import org.junit.jupiter.api.io.TempDirFactory;
 import org.junit.jupiter.engine.AbstractJupiterTestEngineTests;
+import org.junit.platform.launcher.core.LauncherDiscoveryRequestBuilder;
 import org.junit.platform.testkit.engine.EngineExecutionResults;
+import org.junit.platform.testkit.engine.Events;
 
 /**
  * Integration tests for the legacy behavior of the {@link TempDirectory}
@@ -65,13 +69,16 @@ import org.junit.platform.testkit.engine.EngineExecutionResults;
 @DisplayName("TempDirectory extension (per context)")
 class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 
-	@SuppressWarnings("deprecation")
 	@Override
 	protected EngineExecutionResults executeTestsForClass(Class<?> testClass) {
-		return executeTests(request() //
+		return executeTests(requestBuilder(testClass).build());
+	}
+
+	@SuppressWarnings("deprecation")
+	private static LauncherDiscoveryRequestBuilder requestBuilder(Class<?> testClass) {
+		return request() //
 				.selectors(selectClass(testClass)) //
-				.configurationParameter(TempDir.SCOPE_PROPERTY_NAME, TempDirectory.Scope.PER_CONTEXT.toString()) //
-				.build());
+				.configurationParameter(TempDir.SCOPE_PROPERTY_NAME, TempDirectory.Scope.PER_CONTEXT.name());
 	}
 
 	@BeforeEach
@@ -119,7 +126,6 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 	}
 
 	@Test
-	@DisabledOnOs(OS.WINDOWS)
 	@DisplayName("is capable of removing a read-only file in a read-only dir")
 	void readOnlyFileInReadOnlyDirDoesNotCauseFailure() {
 		executeTestsForClass(ReadOnlyFileInReadOnlyDirDoesNotCauseFailureTestCase.class).testEvents()//
@@ -127,7 +133,6 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 	}
 
 	@Test
-	@DisabledOnOs(OS.WINDOWS)
 	@DisplayName("is capable of removing a read-only file in a dir in a read-only dir")
 	void readOnlyFileInNestedReadOnlyDirDoesNotCauseFailure() {
 		executeTestsForClass(ReadOnlyFileInDirInReadOnlyDirDoesNotCauseFailureTestCase.class).testEvents()//
@@ -285,6 +290,43 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 
 	}
 
+	@Nested
+	@DisplayName("supports default factory")
+	@TestMethodOrder(OrderAnnotation.class)
+	class DefaultFactory {
+
+		private Events executeTestsForClassWithDefaultFactory(Class<?> testClass,
+				Class<? extends TempDirFactory> factoryClass) {
+			return TempDirectoryPerContextTests.super.executeTests(requestBuilder(testClass) //
+					.configurationParameter(TempDir.DEFAULT_FACTORY_PROPERTY_NAME, factoryClass.getName()) //
+					.build()).testEvents();
+		}
+
+		@Test
+		@DisplayName("set to Jupiter's default")
+		void supportsStandardDefaultFactory() {
+			executeTestsForClassWithDefaultFactory(StandardDefaultFactoryTestCase.class, TempDirFactory.Standard.class) //
+					.assertStatistics(stats -> stats.started(1).succeeded(1));
+		}
+
+		@Test
+		@DisplayName("set to custom factory")
+		void supportsCustomDefaultFactory() {
+			executeTestsForClassWithDefaultFactory(NonStandardDefaultFactoryTestCase.class, Factory.class) //
+					.assertStatistics(stats -> stats.started(1).succeeded(1));
+		}
+
+		private static class Factory implements TempDirFactory {
+
+			@Override
+			public Path createTempDirectory(AnnotatedElementContext elementContext, ExtensionContext extensionContext)
+					throws Exception {
+				return Files.createTempDirectory("junit");
+			}
+		}
+
+	}
+
 	@Nested
 	@DisplayName("reports failure")
 	@TestMethodOrder(OrderAnnotation.class)
@@ -346,6 +388,23 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 				"@TempDir is not supported on constructor parameters. Please use field injection instead.");
 		}
 
+		@Test
+		@DisplayName("when @TempDir factory is not Standard")
+		@Order(32)
+		void onlySupportsStandardTempDirFactory() {
+			var results = executeTestsForClass(NonStandardFactoryTestCase.class);
+
+			// @formatter:off
+			TempDirectoryPerContextTests.assertSingleFailedTest(results,
+					instanceOf(ParameterResolutionException.class),
+					message(m -> m.matches("Failed to resolve parameter \\[.+] in method \\[.+]: .+")),
+					cause(
+							instanceOf(ExtensionConfigurationException.class),
+							message("Custom @TempDir factory is not supported with junit.jupiter.tempdir.scope=per_context. "
+									+ "Use junit.jupiter.tempdir.factory.default instead.")));
+			// @formatter:on
+		}
+
 	}
 
 	@Nested
@@ -626,6 +685,42 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 		}
 	}
 
+	static class NonStandardFactoryTestCase {
+
+		@Test
+		void test(@SuppressWarnings("unused") @TempDir(factory = Factory.class) Path tempDir) {
+			// never called
+		}
+
+		private static class Factory implements TempDirFactory {
+
+			@Override
+			public Path createTempDirectory(AnnotatedElementContext elementContext, ExtensionContext extensionContext)
+					throws Exception {
+				return Files.createTempDirectory("junit");
+			}
+		}
+
+	}
+
+	static class StandardDefaultFactoryTestCase {
+
+		@Test
+		void test(@TempDir Path tempDir1, @TempDir Path tempDir2) {
+			assertSame(tempDir1, tempDir2);
+		}
+
+	}
+
+	static class NonStandardDefaultFactoryTestCase {
+
+		@Test
+		void test(@TempDir Path tempDir1, @TempDir Path tempDir2) {
+			assertSame(tempDir1, tempDir2);
+		}
+
+	}
+
 	static class AnnotationOnBeforeAllMethodParameterTestCase extends BaseSharedTempDirParameterInjectionTestCase {
 
 		@BeforeAll
@@ -847,8 +942,8 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 		void createReadOnlyFileInReadOnlyDir(@TempDir File tempDir) throws IOException {
 			File file = tempDir.toPath().resolve("file").toFile();
 			assumeTrue(file.createNewFile());
-			assumeTrue(tempDir.setReadOnly());
-			assumeTrue(file.setReadOnly());
+			assumeTrue(makeReadOnly(tempDir));
+			assumeTrue(makeReadOnly(file));
 		}
 
 	}
@@ -861,13 +956,22 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 			File file = tempDir.toPath().resolve("dir").resolve("file").toFile();
 			assumeTrue(file.getParentFile().mkdirs());
 			assumeTrue(file.createNewFile());
-			assumeTrue(tempDir.setReadOnly());
-			assumeTrue(file.getParentFile().setReadOnly());
-			assumeTrue(file.setReadOnly());
+			assumeTrue(makeReadOnly(tempDir));
+			assumeTrue(makeReadOnly(file.getParentFile()));
+			assumeTrue(makeReadOnly(file));
 		}
 
 	}
 
+	private static boolean makeReadOnly(File file) throws IOException {
+		var dos = Files.getFileAttributeView(file.toPath(), DosFileAttributeView.class);
+		if (dos != null) {
+			dos.setReadOnly(true);
+			return true;
+		}
+		return file.setReadOnly();
+	}
+
 	// https://github.com/junit-team/junit5/issues/2609
 	@SuppressWarnings("ResultOfMethodCallIgnored")
 	static class NonMintPermissionContentInTempDirectoryDoesNotCauseFailureTestCase {
@@ -1305,4 +1409,5 @@ class TempDirectoryPerContextTests extends AbstractJupiterTestEngineTests {
 			}
 		}
 	}
+
 }
