@@ -10,7 +10,6 @@
 
 package org.junit.jupiter.params.provider;
 
-import static org.apiguardian.api.API.Status.EXPERIMENTAL;
 import static org.apiguardian.api.API.Status.STABLE;
 
 import java.lang.annotation.Documented;
@@ -68,6 +67,7 @@ import org.apiguardian.api.API;
 @Documented
 @API(status = STABLE, since = "5.7")
 @ArgumentsSource(CsvArgumentsProvider.class)
+@SuppressWarnings("exports")
 public @interface CsvSource {
 
 	/**
@@ -152,7 +152,7 @@ public @interface CsvSource {
 	 * @see #value
 	 * @see #quoteCharacter
 	 */
-	@API(status = EXPERIMENTAL, since = "5.8.1")
+	@API(status = STABLE, since = "5.10")
 	String textBlock() default "";
 
 	/**
@@ -185,7 +185,7 @@ public @interface CsvSource {
 	 *
 	 * @since 5.8.2
 	 */
-	@API(status = EXPERIMENTAL, since = "5.8.2")
+	@API(status = STABLE, since = "5.10")
 	boolean useHeadersInDisplayName() default false;
 
 	/**
@@ -200,7 +200,7 @@ public @interface CsvSource {
 	 * @since 5.8.2
 	 * @see #textBlock
 	 */
-	@API(status = EXPERIMENTAL, since = "5.8.2")
+	@API(status = STABLE, since = "5.10")
 	char quoteCharacter() default '\'';
 
 	/**
@@ -264,7 +264,7 @@ public @interface CsvSource {
 	 *
 	 * @since 5.7
 	 */
-	@API(status = EXPERIMENTAL, since = "5.7")
+	@API(status = STABLE, since = "5.10")
 	int maxCharsPerColumn() default 4096;
 
 	/**
@@ -275,7 +275,7 @@ public @interface CsvSource {
 	 *
 	 * @since 5.8
 	 */
-	@API(status = EXPERIMENTAL, since = "5.8")
+	@API(status = STABLE, since = "5.10")
 	boolean ignoreLeadingAndTrailingWhitespace() default true;
 
 }
