@@ -13,7 +13,7 @@ package org.junit.jupiter.params.provider;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.engine.extension.MutableExtensionRegistry.createRegistryWithDefaultExtensions;
-import static org.junit.jupiter.params.provider.MethodArgumentsProviderTests.DefaultFactoryMethodNameTestCase.TEST_METHOD;
+import static org.junit.platform.commons.util.ReflectionUtils.findMethod;
 import static org.mockito.Mockito.doCallRealMethod;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
@@ -23,6 +23,7 @@ import java.util.Arrays;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Optional;
+import java.util.stream.Collectors;
 import java.util.stream.DoubleStream;
 import java.util.stream.IntStream;
 import java.util.stream.LongStream;
@@ -31,15 +32,16 @@ import java.util.stream.Stream;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Nested;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.TestInstance.Lifecycle;
 import org.junit.jupiter.api.extension.ExtensionContext;
 import org.junit.jupiter.api.extension.ParameterContext;
 import org.junit.jupiter.api.extension.ParameterResolver;
-import org.junit.jupiter.engine.config.JupiterConfiguration;
 import org.junit.jupiter.engine.execution.DefaultExecutableInvoker;
 import org.junit.jupiter.engine.extension.MutableExtensionRegistry;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.platform.commons.JUnitException;
 import org.junit.platform.commons.PreconditionViolationException;
+import org.junit.platform.commons.test.TestClassLoader;
 import org.junit.platform.commons.util.ReflectionUtils;
 
 /**
@@ -49,26 +51,6 @@ class MethodArgumentsProviderTests {
 
 	private MutableExtensionRegistry extensionRegistry;
 
-	@Test
-	void throwsExceptionWhenFactoryMethodDoesNotExist() {
-		var exception = assertThrows(JUnitException.class, () -> provideArguments("unknownMethod").toArray());
-
-		assertThat(exception.getMessage()).isEqualTo(
-			"Could not find factory method [unknownMethod] in class [" + TestCase.class.getName() + "]");
-	}
-
-	@Test
-	void throwsExceptionForIllegalReturnType() {
-		var exception = assertThrows(PreconditionViolationException.class,
-			() -> provideArguments("providerWithIllegalReturnType").toArray());
-
-		assertThat(exception.getMessage())//
-				.containsSubsequence("Could not find valid factory method [providerWithIllegalReturnType] in class [",
-					TestCase.class.getName() + "]", //
-					"but found the following invalid candidates: ", //
-					"[static java.lang.Object org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase.providerWithIllegalReturnType()]");
-	}
-
 	@Test
 	void providesArgumentsUsingStream() {
 		var arguments = provideArguments("stringStreamProvider");
@@ -193,24 +175,57 @@ class MethodArgumentsProviderTests {
 	}
 
 	@Test
-	void throwsExceptionWhenNonStaticFactoryMethodIsReferencedAndStaticIsRequired() {
-		var exception = assertThrows(JUnitException.class,
-			() -> provideArguments(NonStaticTestCase.class, null, false, "nonStaticStringStreamProvider").toArray());
+	void throwsExceptionWhenNonStaticLocalFactoryMethodIsReferencedWithLifecyclePerMethodSemantics() {
+		var lifecyclePerClass = false;
+		var exception = assertThrows(PreconditionViolationException.class,
+			() -> provideArguments(NonStaticTestCase.class, lifecyclePerClass,
+				"nonStaticStringStreamProvider").toArray());
 
-		assertThat(exception).hasMessageContaining("Cannot invoke non-static method");
+		assertStaticIsRequired(exception);
+	}
+
+	@Test
+	void throwsExceptionWhenNonStaticExternalFactoryMethodIsReferencedWithLifecyclePerMethodSemantics() {
+		var factoryClass = NonStaticTestCase.class.getName();
+		var factoryMethod = factoryClass + "#nonStaticStringStreamProvider";
+		var lifecyclePerClass = false;
+		var exception = assertThrows(PreconditionViolationException.class,
+			() -> provideArguments(TestCase.class, lifecyclePerClass, factoryMethod).toArray());
+
+		assertStaticIsRequired(exception);
+	}
+
+	@Test
+	void throwsExceptionWhenNonStaticExternalFactoryMethodIsReferencedWithLifecyclePerClassSemantics() {
+		var factoryClass = NonStaticTestCase.class.getName();
+		var factoryMethod = factoryClass + "#nonStaticStringStreamProvider";
+		boolean lifecyclePerClass = true;
+		var exception = assertThrows(PreconditionViolationException.class,
+			() -> provideArguments(TestCase.class, lifecyclePerClass, factoryMethod).toArray());
+
+		assertStaticIsRequired(exception);
+	}
+
+	private static void assertStaticIsRequired(PreconditionViolationException exception) {
+		assertThat(exception).hasMessageContainingAll("Method '",
+			"' must be static: local factory methods must be static ",
+			"unless the PER_CLASS @TestInstance lifecycle mode is used; ",
+			"external factory methods must always be static.");
 	}
 
 	@Test
 	void providesArgumentsFromNonStaticFactoryMethodWhenStaticIsNotRequired() {
-		var arguments = provideArguments(NonStaticTestCase.class, null, true, "nonStaticStringStreamProvider");
+		var arguments = provideArguments(NonStaticTestCase.class, true, "nonStaticStringStreamProvider");
 
 		assertThat(arguments).containsExactly(array("foo"), array("bar"));
 	}
 
 	@Test
-	void providesArgumentsUsingDefaultFactoryMethodName() throws Exception {
-		Class<?> testClass = DefaultFactoryMethodNameTestCase.class;
-		var testMethod = testClass.getDeclaredMethod(TEST_METHOD, String.class);
+	void providesArgumentsUsingDefaultFactoryMethodName() {
+		var testClass = DefaultFactoryMethodNameTestCase.class;
+		var methodName = "testDefaultFactoryMethodName";
+		var testMethod = findMethod(testClass, methodName, String.class).get();
+
 		var arguments = provideArguments(testClass, testMethod, false, "");
 
 		assertThat(arguments).containsExactly(array("foo"), array("bar"));
@@ -223,6 +238,31 @@ class MethodArgumentsProviderTests {
 		assertThat(arguments).containsExactly(array("string1"), array("string2"));
 	}
 
+	@Test
+	void providesArgumentsUsingExternalFactoryMethodInTypeFromDifferentClassLoader() throws Exception {
+		try (var testClassLoader = TestClassLoader.forClasses(TestCase.class, ExternalFactoryMethods.class)) {
+			var testClass = testClassLoader.loadClass(TestCase.class.getName());
+			var testMethod = ReflectionUtils.findMethod(testClass, "test").get();
+			var fullyQualifiedMethodName = ExternalFactoryMethods.class.getName() + "#stringsProvider";
+
+			assertThat(testClass.getClassLoader()).isSameAs(testClassLoader);
+
+			var arguments = provideArguments(testClass, false, fullyQualifiedMethodName);
+			assertThat(arguments).containsExactly(array("string1"), array("string2"));
+
+			var factoryMethod = MethodArgumentsProvider.findFactoryMethodByFullyQualifiedName(testClass, testMethod,
+				fullyQualifiedMethodName);
+			assertThat(factoryMethod).isNotNull();
+			assertThat(factoryMethod.getName()).isEqualTo("stringsProvider");
+			assertThat(factoryMethod.getParameterTypes()).isEmpty();
+
+			var declaringClass = factoryMethod.getDeclaringClass();
+			assertThat(declaringClass.getName()).isEqualTo(ExternalFactoryMethods.class.getName());
+			assertThat(declaringClass).isNotEqualTo(ExternalFactoryMethods.class);
+			assertThat(declaringClass.getClassLoader()).isSameAs(testClassLoader);
+		}
+	}
+
 	@Test
 	void providesArgumentsUsingExternalFactoryMethodWithParentheses() {
 		var arguments = provideArguments(ExternalFactoryMethods.class.getName() + "#stringsProvider()");
@@ -245,35 +285,6 @@ class MethodArgumentsProviderTests {
 		assertThat(arguments).containsExactly(array("foo"), array("bar"), array("string1"), array("string2"));
 	}
 
-	@Test
-	void throwsExceptionWhenClassForExternalFactoryMethodCannotBeLoaded() {
-		var exception = assertThrows(JUnitException.class,
-			() -> provideArguments("com.example.NonExistentExternalFactoryMethods#stringsProvider").toArray());
-
-		assertThat(exception.getMessage()).isEqualTo(
-			"Could not load class [com.example.NonExistentExternalFactoryMethods]");
-	}
-
-	@Test
-	void throwsExceptionWhenExternalFactoryMethodCannotBeFound() {
-		var exception = assertThrows(JUnitException.class,
-			() -> provideArguments(ExternalFactoryMethods.class.getName() + "#nonExistentMethod").toArray());
-
-		assertThat(exception.getMessage()).isEqualTo("Could not find factory method [nonExistentMethod()] in class ["
-				+ ExternalFactoryMethods.class.getName() + "]");
-	}
-
-	@Test
-	void throwsExceptionWhenFullyQualifiedMethodNameIsInvalid() {
-		var exception = assertThrows(JUnitException.class,
-			() -> provideArguments(ExternalFactoryMethods.class.getName() + ".wrongSyntax").toArray());
-
-		assertThat(exception.getMessage()).isEqualTo(
-			"[" + ExternalFactoryMethods.class.getName() + ".wrongSyntax] is not a valid fully qualified method name: "
-					+ "it must start with a fully qualified class name followed by a '#' and then the method name, "
-					+ "optionally followed by a parameter list enclosed in parentheses.");
-	}
-
 	@Nested
 	class PrimitiveArrays {
 
@@ -364,31 +375,41 @@ class MethodArgumentsProviderTests {
 	@Nested
 	class ParameterResolution {
 
+		private final Method testMethod = findMethod(TestCase.class, "test").get();
+
 		@BeforeEach
 		void registerParameterResolver() {
-			JupiterConfiguration configuration = mock(JupiterConfiguration.class);
-			extensionRegistry = createRegistryWithDefaultExtensions(configuration);
+			extensionRegistry = createRegistryWithDefaultExtensions(mock());
 			extensionRegistry.registerExtension(StringResolver.class);
+			extensionRegistry.registerExtension(StringArrayResolver.class);
+			extensionRegistry.registerExtension(IntArrayResolver.class);
 		}
 
 		@Test
-		void providesArgumentsUsingDefaultFactoryMethodWithParameter() throws Exception {
-			Class<?> testClass = TestCase.class;
-			var testMethod = testClass.getDeclaredMethod("overloadedStringStreamProvider", Object.class);
-			var arguments = provideArguments(testClass, testMethod, false, "");
+		void providesArgumentsInferringDefaultFactoryMethodThatAcceptsArgument() {
+			Method testMethod = findMethod(TestCase.class, "overloadedStringStreamProvider", Object.class).get();
+			String factoryMethodName = ""; // signals to use default
+			var arguments = provideArguments(testMethod, factoryMethodName);
 
 			assertThat(arguments).containsExactly(array("foo!"), array("bar!"));
 		}
 
 		@Test
-		void providesArgumentsUsingFactoryMethodWithParameter() {
+		void providesArgumentsUsingSimpleNameForFactoryMethodThatAcceptsArgumentWithoutSpecifyingParameterList() {
 			var arguments = provideArguments("stringStreamProviderWithParameter");
 
 			assertThat(arguments).containsExactly(array("foo!"), array("bar!"));
 		}
 
 		@Test
-		void providesArgumentsUsingFullyQualifiedNameWithParameter() {
+		void providesArgumentsUsingFullyQualifiedNameForFactoryMethodThatAcceptsArgumentWithoutSpecifyingParameterList() {
+			var arguments = provideArguments(TestCase.class.getName() + "#stringStreamProviderWithParameter");
+
+			assertThat(arguments).containsExactly(array("foo!"), array("bar!"));
+		}
+
+		@Test
+		void providesArgumentsUsingFullyQualifiedNameSpecifyingParameter() {
 			var arguments = provideArguments(
 				TestCase.class.getName() + "#stringStreamProviderWithParameter(java.lang.String)");
 
@@ -396,66 +417,310 @@ class MethodArgumentsProviderTests {
 		}
 
 		@Test
-		void providesArgumentsUsingSimpleNameWithoutParameter() {
-			var arguments = provideArguments("stringStreamProviderWithOrWithoutParameter()");
+		void providesArgumentsUsingLocalQualifiedNameSpecifyingParameter() {
+			var arguments = provideArguments(testMethod, "stringStreamProviderWithParameter(java.lang.String)");
+
+			assertThat(arguments).containsExactly(array("foo!"), array("bar!"));
+		}
+
+		@Test
+		void providesArgumentsUsingFullyQualifiedNameForOverloadedFactoryMethodSpecifyingEmptyParameterList() {
+			var arguments = provideArguments(
+				TestCase.class.getName() + "#stringStreamProviderWithOrWithoutParameter()");
 
 			assertThat(arguments).containsExactly(array("foo"), array("bar"));
 		}
 
 		@Test
-		void providesArgumentsUsingSimpleNameWithParameter() {
-			var arguments = provideArguments("stringStreamProviderWithOrWithoutParameter(java.lang.String)");
+		void providesArgumentsUsingLocalQualifiedNameForOverloadedFactoryMethodSpecifyingEmptyParameterList() {
+			var arguments = provideArguments(this.testMethod, "stringStreamProviderWithOrWithoutParameter()");
 
-			assertThat(arguments).containsExactly(array("foo!"), array("bar!"));
-		}
-
-		@ParameterizedTest
-		@ValueSource(strings = { "java.lang.String,java.lang.String", "java.lang.String, java.lang.String",
-				"java.lang.String,    java.lang.String" })
-		void providesArgumentsUsingSimpleNameWithMultipleParameters(String params) {
-			var arguments = provideArguments("stringStreamProviderWithOrWithoutParameter(" + params + ")");
-			assertThat(arguments).containsExactly(array("foo!!"), array("bar!!"));
+			assertThat(arguments).containsExactly(array("foo"), array("bar"));
 		}
 
 		@Test
-		void throwsExceptionWhenSeveralFactoryMethodsWithSameNameAreAvailable() {
-			var exception = assertThrows(PreconditionViolationException.class,
-				() -> provideArguments("stringStreamProviderWithOrWithoutParameter").toArray());
-
-			assertThat(exception.getMessage())//
-					.startsWith("3 factory methods named [stringStreamProviderWithOrWithoutParameter] were found in "
-							+ "class [org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase]: ")//
-					.contains("stringStreamProviderWithOrWithoutParameter()",
-						"stringStreamProviderWithOrWithoutParameter(java.lang.String)",
-						"stringStreamProviderWithOrWithoutParameter(java.lang.String,java.lang.String)");
-		}
-
-		@Test
-		void providesArgumentsUsingFactoryMethodSelectedViaFullyQualifiedNameWithParameter() {
+		void providesArgumentsUsingFullyQualifiedNameForOverloadedFactoryMethodSpecifyingParameter() {
 			var arguments = provideArguments(
 				TestCase.class.getName() + "#stringStreamProviderWithOrWithoutParameter(java.lang.String)");
 
 			assertThat(arguments).containsExactly(array("foo!"), array("bar!"));
 		}
 
+		@Test
+		void providesArgumentsUsingLocalQualifiedNameForOverloadedFactoryMethodSpecifyingParameter() {
+			var arguments = provideArguments(testMethod,
+				"stringStreamProviderWithOrWithoutParameter(java.lang.String)");
+
+			assertThat(arguments).containsExactly(array("foo!"), array("bar!"));
+		}
+
+		@Test
+		void failsToProvideArgumentsUsingFullyQualifiedNameSpecifyingInvalidParameterType() {
+			String method = TestCase.class.getName() + "#stringStreamProviderWithParameter(example.FooBar)";
+			var exception = assertThrows(JUnitException.class, () -> provideArguments(method).toArray());
+
+			assertThat(exception).hasMessage("""
+					Failed to load parameter type [example.FooBar] for method [stringStreamProviderWithParameter] \
+					in class [org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase].""");
+		}
+
+		@Test
+		void failsToProvideArgumentsUsingLocalQualifiedNameSpecifyingInvalidParameterType() {
+			var method = "stringStreamProviderWithParameter(example.FooBar)";
+			var exception = assertThrows(JUnitException.class,
+				() -> provideArguments(this.testMethod, method).toArray());
+
+			assertThat(exception).hasMessage("""
+					Failed to load parameter type [example.FooBar] for method [stringStreamProviderWithParameter] \
+					in class [org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase].""");
+		}
+
+		@Test
+		void failsToProvideArgumentsUsingFullyQualifiedNameSpecifyingIncorrectParameterType() {
+			String method = TestCase.class.getName() + "#stringStreamProviderWithParameter(java.lang.Integer)";
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(method).toArray());
+
+			assertThat(exception).hasMessage("""
+					Could not find factory method [stringStreamProviderWithParameter(java.lang.Integer)] in \
+					class [org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase]""");
+		}
+
+		@Test
+		void failsToProvideArgumentsUsingLocalQualifiedNameSpecifyingIncorrectParameterType() {
+			var method = "stringStreamProviderWithParameter(java.lang.Integer)";
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(this.testMethod, method).toArray());
+
+			assertThat(exception).hasMessage("""
+					Could not find factory method [stringStreamProviderWithParameter(java.lang.Integer)] in \
+					class [org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase]""");
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = {
+				"org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase#stringStreamProviderWithArrayParameter(java.lang.String[])",
+				"org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase#stringStreamProviderWithArrayParameter([Ljava.lang.String;)", })
+		void providesArgumentsUsingFullyQualifiedNameSpecifyingObjectArrayParameter(String method) {
+			var arguments = provideArguments(method);
+
+			assertThat(arguments).containsExactly(array("foo :)"), array("bar :)"));
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = { //
+				"stringStreamProviderWithArrayParameter(java.lang.String[])",
+				"stringStreamProviderWithArrayParameter([Ljava.lang.String;)" })
+		void providesArgumentsUsingLocalQualifiedNameSpecifyingObjectArrayParameter(String method) {
+			var arguments = provideArguments(this.testMethod, method);
+
+			assertThat(arguments).containsExactly(array("foo :)"), array("bar :)"));
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = {
+				"org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase#stringStreamProviderWithArrayParameter(int[])",
+				"org.junit.jupiter.params.provider.MethodArgumentsProviderTests$TestCase#stringStreamProviderWithArrayParameter([I)", })
+		void providesArgumentsUsingFullyQualifiedNameSpecifyingPrimitiveArrayParameter(String method) {
+			var arguments = provideArguments(method);
+
+			assertThat(arguments).containsExactly(array("foo 42"), array("bar 42"));
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = { //
+				"stringStreamProviderWithArrayParameter(int[])", //
+				"stringStreamProviderWithArrayParameter([I)" })
+		void providesArgumentsUsingLocalQualifiedNameSpecifyingPrimitiveArrayParameter(String method) {
+			var arguments = provideArguments(this.testMethod, method);
+
+			assertThat(arguments).containsExactly(array("foo 42"), array("bar 42"));
+		}
+
 		@ParameterizedTest
 		@ValueSource(strings = { "java.lang.String,java.lang.String", "java.lang.String, java.lang.String",
 				"java.lang.String,    java.lang.String" })
-		void providesArgumentsUsingFactoryMethodSelectedViaFullyQualifiedNameWithMultipleParameters(String params) {
-			var arguments = provideArguments(
-				TestCase.class.getName() + "#stringStreamProviderWithOrWithoutParameter(" + params + ")");
+		void providesArgumentsUsingFullyQualifiedNameSpecifyingMultipleParameters(String params) {
+			var method = TestCase.class.getName() + "#stringStreamProviderWithOrWithoutParameter(" + params + ")";
+			var arguments = provideArguments(method);
+
+			assertThat(arguments).containsExactly(array("foo!!"), array("bar!!"));
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = { "java.lang.String,java.lang.String", "java.lang.String, java.lang.String",
+				"java.lang.String,    java.lang.String" })
+		void providesArgumentsUsingLocalQualifiedNameSpecifyingMultipleParameters(String params) {
+			var arguments = provideArguments(this.testMethod,
+				"stringStreamProviderWithOrWithoutParameter(" + params + ")");
 
 			assertThat(arguments).containsExactly(array("foo!!"), array("bar!!"));
 		}
 
 		@Test
-		void providesArgumentsUsingFactoryMethodSelectedViaFullyQualifiedNameWithoutParameter() {
-			var arguments = provideArguments(
-				TestCase.class.getName() + "#stringStreamProviderWithOrWithoutParameter()");
+		void providesArgumentsUsingFullyQualifiedNameForOverloadedFactoryMethodWhenParameterListIsNotSpecified() {
+			var arguments = provideArguments(TestCase.class.getName() + "#stringStreamProviderWithOrWithoutParameter");
 
 			assertThat(arguments).containsExactly(array("foo"), array("bar"));
 		}
 
+		@Test
+		void providesArgumentsUsingLocalQualifiedNameForOverloadedFactoryMethodWhenParameterListIsNotSpecified() {
+			var arguments = provideArguments("stringStreamProviderWithOrWithoutParameter").toArray();
+
+			assertThat(arguments).containsExactly(array("foo"), array("bar"));
+		}
+
+	}
+
+	@Nested
+	class ErrorCases {
+
+		@Test
+		void throwsExceptionWhenFullyQualifiedMethodNameSyntaxIsInvalid() {
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments("org.example.wrongSyntax").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo(
+				"[org.example.wrongSyntax] is not a valid fully qualified method name: "
+						+ "it must start with a fully qualified class name followed by a '#' and then the method name, "
+						+ "optionally followed by a parameter list enclosed in parentheses.");
+		}
+
+		@Test
+		void throwsExceptionWhenClassForExternalFactoryMethodCannotBeLoaded() {
+			var exception = assertThrows(JUnitException.class,
+				() -> provideArguments("com.example.NonExistentClass#stringsProvider").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo("Could not load class [com.example.NonExistentClass]");
+		}
+
+		@Test
+		void throwsExceptionWhenExternalFactoryMethodDoesNotExist() {
+			String factoryClass = ExternalFactoryMethods.class.getName();
+
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(factoryClass + "#nonExistentMethod").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo(
+				"Could not find factory method [nonExistentMethod] in class [%s]", factoryClass);
+		}
+
+		@Test
+		void throwsExceptionWhenLocalFactoryMethodDoesNotExist() {
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments("nonExistentMethod").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo(
+				"Could not find factory method [nonExistentMethod] in class [%s]", TestCase.class.getName());
+		}
+
+		@Test
+		void throwsExceptionWhenExternalFactoryMethodAcceptingSingleArgumentDoesNotExist() {
+			String factoryClass = ExternalFactoryMethods.class.getName();
+
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(factoryClass + "#nonExistentMethod(int)").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo(
+				"Could not find factory method [nonExistentMethod(int)] in class [%s]", factoryClass);
+		}
+
+		@Test
+		void throwsExceptionWhenLocalFactoryMethodAcceptingSingleArgumentDoesNotExist() {
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments("nonExistentMethod(int)").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo(
+				"Could not find factory method [nonExistentMethod(int)] in class [%s]", TestCase.class.getName());
+		}
+
+		@Test
+		void throwsExceptionWhenExternalFactoryMethodAcceptingMultipleArgumentsDoesNotExist() {
+			String factoryClass = ExternalFactoryMethods.class.getName();
+
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(factoryClass + "#nonExistentMethod(int, java.lang.String)").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo(
+				"Could not find factory method [nonExistentMethod(int, java.lang.String)] in class [%s]", factoryClass);
+		}
+
+		@Test
+		void throwsExceptionWhenLocalFactoryMethodAcceptingMultipleArgumentsDoesNotExist() {
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments("nonExistentMethod(java.lang.String,int)").toArray());
+
+			assertThat(exception.getMessage()).isEqualTo(
+				"Could not find factory method [nonExistentMethod(java.lang.String,int)] in class [%s]",
+				TestCase.class.getName());
+		}
+
+		@Test
+		void throwsExceptionWhenExternalFactoryMethodHasInvalidReturnType() {
+			String testClass = TestCase.class.getName();
+			String factoryClass = ExternalFactoryMethods.class.getName();
+			String factoryMethod = factoryClass + "#factoryWithInvalidReturnType";
+
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(TestCase.class, false, factoryMethod).toArray());
+
+			assertThat(exception.getMessage())//
+					.containsSubsequence("Could not find valid factory method [" + factoryMethod + "] for test class [",
+						testClass + "]", //
+						"but found the following invalid candidate: ",
+						"static java.lang.Object " + factoryClass + ".factoryWithInvalidReturnType()");
+		}
+
+		@Test
+		void throwsExceptionWhenLocalFactoryMethodHasInvalidReturnType() {
+			String testClass = TestCase.class.getName();
+			String factoryClass = testClass;
+			String factoryMethod = "factoryWithInvalidReturnType";
+
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(factoryMethod).toArray());
+
+			assertThat(exception.getMessage())//
+					.containsSubsequence("Could not find valid factory method [" + factoryMethod + "] for test class [",
+						factoryClass + "]", //
+						"but found the following invalid candidate: ", //
+						"static java.lang.Object " + factoryClass + ".factoryWithInvalidReturnType()");
+		}
+
+		@Test
+		void throwsExceptionWhenMultipleDefaultFactoryMethodCandidatesExist() {
+			var testClass = MultipleDefaultFactoriesTestCase.class;
+			var methodName = "test";
+			var testMethod = findMethod(testClass, methodName, String.class).get();
+
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(testClass, testMethod, false, "").toArray());
+
+			assertThat(exception.getMessage()).contains(//
+				"2 factory methods named [test] were found in class [", testClass.getName() + "]: ", //
+				"$MultipleDefaultFactoriesTestCase.test()", //
+				"$MultipleDefaultFactoriesTestCase.test(int)"//
+			);
+		}
+
+		@Test
+		void throwsExceptionWhenMultipleInvalidDefaultFactoryMethodCandidatesExist() {
+			var testClass = MultipleInvalidDefaultFactoriesTestCase.class;
+			var methodName = "test";
+			var testMethod = findMethod(testClass, methodName, String.class).get();
+
+			var exception = assertThrows(PreconditionViolationException.class,
+				() -> provideArguments(testClass, testMethod, false, "").toArray());
+
+			assertThat(exception.getMessage()).contains(//
+				"Could not find valid factory method [test] in class [", testClass.getName() + "]", //
+				"but found the following invalid candidates: ", //
+				"$MultipleInvalidDefaultFactoriesTestCase.test()", //
+				"$MultipleInvalidDefaultFactoriesTestCase.test(int)"//
+			);
+		}
+
 	}
 
 	// -------------------------------------------------------------------------
@@ -464,26 +729,29 @@ class MethodArgumentsProviderTests {
 		return objects;
 	}
 
-	private Stream<Object[]> provideArguments(String... methodNames) {
-		return provideArguments(TestCase.class, null, false, methodNames);
+	private Stream<Object[]> provideArguments(String... factoryMethodNames) {
+		return provideArguments(TestCase.class, false, factoryMethodNames);
+	}
+
+	private Stream<Object[]> provideArguments(Method testMethod, String factoryMethodName) {
+		return provideArguments(TestCase.class, testMethod, false, factoryMethodName);
+	}
+
+	private Stream<Object[]> provideArguments(Class<?> testClass, boolean allowNonStaticMethod,
+			String... factoryMethodNames) {
+
+		// Ensure we have a non-null test method, even if it's not a real test method.
+		// If this throws an exception, make sure that the supplied test class defines a "void test()" method.
+		Method testMethod = ReflectionUtils.findMethod(testClass, "test").get();
+		return provideArguments(testClass, testMethod, allowNonStaticMethod, factoryMethodNames);
 	}
 
 	private Stream<Object[]> provideArguments(Class<?> testClass, Method testMethod, boolean allowNonStaticMethod,
-			String... methodNames) {
-
-		if (testMethod == null) {
-			try {
-				// ensure we have a non-null method, even if it's not a real test method.
-				testMethod = getClass().getMethod("toString");
-			}
-			catch (Exception ex) {
-				throw new RuntimeException(ex);
-			}
-		}
+			String... factoryMethodNames) {
 
 		var methodSource = mock(MethodSource.class);
 
-		when(methodSource.value()).thenReturn(methodNames);
+		when(methodSource.value()).thenReturn(factoryMethodNames);
 
 		var extensionContext = mock(ExtensionContext.class);
 		when(extensionContext.getTestClass()).thenReturn(Optional.of(testClass));
@@ -497,6 +765,9 @@ class MethodArgumentsProviderTests {
 		var testInstance = allowNonStaticMethod ? ReflectionUtils.newInstance(testClass) : null;
 		when(extensionContext.getTestInstance()).thenReturn(Optional.ofNullable(testInstance));
 
+		var lifeCycle = allowNonStaticMethod ? Lifecycle.PER_CLASS : Lifecycle.PER_METHOD;
+		when(extensionContext.getTestInstanceLifecycle()).thenReturn(Optional.of(lifeCycle));
+
 		var provider = new MethodArgumentsProvider();
 		provider.accept(methodSource);
 		return provider.provideArguments(extensionContext).map(Arguments::get);
@@ -506,21 +777,58 @@ class MethodArgumentsProviderTests {
 
 	static class DefaultFactoryMethodNameTestCase {
 
-		static final String TEST_METHOD = "testDefaultFactoryMethodName";
+		// Test
+		void testDefaultFactoryMethodName(String param) {
+		}
 
+		// Factory
 		static Stream<String> testDefaultFactoryMethodName() {
 			return Stream.of("foo", "bar");
 		}
+	}
 
-		void testDefaultFactoryMethodName(String param) {
+	static class MultipleDefaultFactoriesTestCase {
+
+		// Test
+		void test(String param) {
+		}
+
+		// Factory
+		static Stream<String> test() {
+			return Stream.of();
+		}
+
+		// Another Factory
+		static Stream<Integer> test(int num) {
+			return Stream.of();
+		}
+	}
+
+	static class MultipleInvalidDefaultFactoriesTestCase {
+
+		// Test
+		void test(String param) {
+		}
+
+		// NOT a Factory
+		static String test() {
+			return null;
+		}
+
+		// Also NOT a Factory
+		static Object test(int num) {
+			return null;
 		}
 	}
 
 	static class TestCase {
 
+		void test() {
+		}
+
 		// --- Invalid ---------------------------------------------------------
 
-		static Object providerWithIllegalReturnType() {
+		static Object factoryWithInvalidReturnType() {
 			return -1;
 		}
 
@@ -534,6 +842,16 @@ class MethodArgumentsProviderTests {
 			return Stream.of("foo" + parameter, "bar" + parameter);
 		}
 
+		static Stream<String> stringStreamProviderWithArrayParameter(String[] parameter) {
+			String suffix = Arrays.stream(parameter).collect(Collectors.joining());
+			return Stream.of("foo " + suffix, "bar " + suffix);
+		}
+
+		static Stream<String> stringStreamProviderWithArrayParameter(int[] parameter) {
+			return stringStreamProviderWithArrayParameter(
+				Arrays.stream(parameter).mapToObj(String::valueOf).toArray(String[]::new));
+		}
+
 		static Stream<String> stringStreamProviderWithOrWithoutParameter() {
 			return stringStreamProvider();
 		}
@@ -546,6 +864,10 @@ class MethodArgumentsProviderTests {
 			return stringStreamProviderWithParameter(parameter1 + parameter2);
 		}
 
+		// Overloaded method, but not a valid return type for a factory method
+		static void stringStreamProviderWithOrWithoutParameter(String parameter1, int parameter2) {
+		}
+
 		// @ParameterizedTest
 		// @MethodSource // use default, inferred factory method
 		void overloadedStringStreamProvider(Object parameter) {
@@ -666,6 +988,9 @@ class MethodArgumentsProviderTests {
 	// This test case mimics @TestInstance(Lifecycle.PER_CLASS)
 	static class NonStaticTestCase {
 
+		void test() {
+		}
+
 		Stream<String> nonStaticStringStreamProvider() {
 			return Stream.of("foo", "bar");
 		}
@@ -673,6 +998,10 @@ class MethodArgumentsProviderTests {
 
 	static class ExternalFactoryMethods {
 
+		static Object factoryWithInvalidReturnType() {
+			return -1;
+		}
+
 		static Stream<String> stringsProvider() {
 			return Stream.of("string1", "string2");
 		}
@@ -698,4 +1027,30 @@ class MethodArgumentsProviderTests {
 		}
 	}
 
+	static class StringArrayResolver implements ParameterResolver {
+
+		@Override
+		public boolean supportsParameter(ParameterContext parameterContext, ExtensionContext extensionContext) {
+			return parameterContext.getParameter().getType() == String[].class;
+		}
+
+		@Override
+		public String[] resolveParameter(ParameterContext parameterContext, ExtensionContext extensionContext) {
+			return new String[] { ":", ")" };
+		}
+	}
+
+	static class IntArrayResolver implements ParameterResolver {
+
+		@Override
+		public boolean supportsParameter(ParameterContext parameterContext, ExtensionContext extensionContext) {
+			return parameterContext.getParameter().getType() == int[].class;
+		}
+
+		@Override
+		public int[] resolveParameter(ParameterContext parameterContext, ExtensionContext extensionContext) {
+			return new int[] { 4, 2 };
+		}
+	}
+
 }
