@@ -10,8 +10,8 @@
 
 package org.junit.platform.commons.support;
 
-import static org.apiguardian.api.API.Status.EXPERIMENTAL;
 import static org.apiguardian.api.API.Status.MAINTAINED;
+import static org.apiguardian.api.API.Status.STABLE;
 
 import java.lang.annotation.Annotation;
 import java.lang.reflect.AnnotatedElement;
@@ -154,7 +154,7 @@ public final class AnnotationSupport {
 	 * @see SearchOption
 	 * @see #findAnnotation(AnnotatedElement, Class)
 	 */
-	@API(status = EXPERIMENTAL, since = "1.8")
+	@API(status = STABLE, since = "1.10")
 	public static <A extends Annotation> Optional<A> findAnnotation(Class<?> clazz, Class<A> annotationType,
 			SearchOption searchOption) {
 
@@ -285,7 +285,36 @@ public final class AnnotationSupport {
 	 */
 	@API(status = MAINTAINED, since = "1.4")
 	public static List<Field> findAnnotatedFields(Class<?> clazz, Class<? extends Annotation> annotationType) {
-		return AnnotationUtils.findAnnotatedFields(clazz, annotationType, field -> true);
+		return findAnnotatedFields(clazz, annotationType, field -> true);
+	}
+
+	/**
+	 * Find all {@linkplain Field fields} of the supplied class or interface
+	 * that are annotated or <em>meta-annotated</em> with the specified
+	 * {@code annotationType} and match the specified {@code predicate}, using
+	 * top-down search semantics within the type hierarchy.
+	 *
+	 * <p>Fields declared in the same class or interface will be ordered using
+	 * an algorithm that is deterministic but intentionally nonobvious.
+	 *
+	 * <p>The results will not contain fields that are <em>hidden</em> or
+	 * {@linkplain Field#isSynthetic() synthetic}.
+	 *
+	 * @param clazz the class or interface in which to find the fields; never {@code null}
+	 * @param annotationType the annotation type to search for; never {@code null}
+	 * @param predicate the field filter; never {@code null}
+	 * @return the list of all such fields found; neither {@code null} nor mutable
+	 * @since 1.10
+	 * @see Class#getDeclaredFields()
+	 * @see #findPublicAnnotatedFields(Class, Class, Class)
+	 * @see #findAnnotatedFields(Class, Class, Predicate, HierarchyTraversalMode)
+	 * @see ReflectionSupport#findFields(Class, Predicate, HierarchyTraversalMode)
+	 * @see ReflectionSupport#tryToReadFieldValue(Field, Object)
+	 */
+	@API(status = MAINTAINED, since = "1.10")
+	public static List<Field> findAnnotatedFields(Class<?> clazz, Class<? extends Annotation> annotationType,
+			Predicate<Field> predicate) {
+		return AnnotationUtils.findAnnotatedFields(clazz, annotationType, predicate);
 	}
 
 	/**
