@@ -24,12 +24,14 @@ import java.util.function.Supplier;
 import org.apiguardian.api.API;
 import org.junit.platform.commons.JUnitException;
 import org.junit.platform.commons.util.ClassLoaderUtils;
-import org.junit.platform.console.options.CommandLineOptions;
 import org.junit.platform.console.options.Details;
+import org.junit.platform.console.options.TestConsoleOutputOptions;
+import org.junit.platform.console.options.TestDiscoveryOptions;
 import org.junit.platform.console.options.Theme;
 import org.junit.platform.launcher.Launcher;
 import org.junit.platform.launcher.LauncherDiscoveryRequest;
 import org.junit.platform.launcher.TestExecutionListener;
+import org.junit.platform.launcher.TestPlan;
 import org.junit.platform.launcher.core.LauncherFactory;
 import org.junit.platform.launcher.listeners.SummaryGeneratingListener;
 import org.junit.platform.launcher.listeners.TestExecutionSummary;
@@ -41,32 +43,66 @@ import org.junit.platform.reporting.legacy.xml.LegacyXmlReportGeneratingListener
 @API(status = INTERNAL, since = "1.0")
 public class ConsoleTestExecutor {
 
-	private final CommandLineOptions options;
+	private final TestDiscoveryOptions discoveryOptions;
+	private final TestConsoleOutputOptions outputOptions;
 	private final Supplier<Launcher> launcherSupplier;
 
-	public ConsoleTestExecutor(CommandLineOptions options) {
-		this(options, LauncherFactory::create);
+	public ConsoleTestExecutor(TestDiscoveryOptions discoveryOptions, TestConsoleOutputOptions outputOptions) {
+		this(discoveryOptions, outputOptions, LauncherFactory::create);
 	}
 
 	// for tests only
-	ConsoleTestExecutor(CommandLineOptions options, Supplier<Launcher> launcherSupplier) {
-		this.options = options;
+	ConsoleTestExecutor(TestDiscoveryOptions discoveryOptions, TestConsoleOutputOptions outputOptions,
+			Supplier<Launcher> launcherSupplier) {
+		this.discoveryOptions = discoveryOptions;
+		this.outputOptions = outputOptions;
 		this.launcherSupplier = launcherSupplier;
 	}
 
-	public TestExecutionSummary execute(PrintWriter out) throws Exception {
-		return new CustomContextClassLoaderExecutor(createCustomClassLoader()).invoke(() -> executeTests(out));
+	public void discover(PrintWriter out) {
+		new CustomContextClassLoaderExecutor(createCustomClassLoader()).invoke(() -> {
+			discoverTests(out);
+			return null;
+		});
 	}
 
-	private TestExecutionSummary executeTests(PrintWriter out) {
-		Launcher launcher = launcherSupplier.get();
-		SummaryGeneratingListener summaryListener = registerListeners(out, launcher);
+	public TestExecutionSummary execute(PrintWriter out, Optional<Path> reportsDir) {
+		return new CustomContextClassLoaderExecutor(createCustomClassLoader()) //
+				.invoke(() -> executeTests(out, reportsDir));
+	}
 
-		LauncherDiscoveryRequest discoveryRequest = new DiscoveryRequestCreator().toDiscoveryRequest(options);
+	private void discoverTests(PrintWriter out) {
+		Launcher launcher = launcherSupplier.get();
+		Optional<DetailsPrintingListener> commandLineTestPrinter = createDetailsPrintingListener(out);
+
+		LauncherDiscoveryRequest discoveryRequest = new DiscoveryRequestCreator().toDiscoveryRequest(discoveryOptions);
+		TestPlan testPlan = launcher.discover(discoveryRequest);
+
+		commandLineTestPrinter.ifPresent(printer -> printer.listTests(testPlan));
+		if (outputOptions.getDetails() != Details.NONE) {
+			printFoundTestsSummary(out, testPlan);
+		}
+	}
+
+	private static void printFoundTestsSummary(PrintWriter out, TestPlan testPlan) {
+		SummaryGeneratingListener summaryListener = new SummaryGeneratingListener();
+		summaryListener.testPlanExecutionStarted(testPlan);
+		TestExecutionSummary summary = summaryListener.getSummary();
+
+		out.printf("%n[%10d containers found ]%n[%10d tests found      ]%n%n", summary.getContainersFoundCount(),
+			summary.getTestsFoundCount());
+		out.flush();
+	}
+
+	private TestExecutionSummary executeTests(PrintWriter out, Optional<Path> reportsDir) {
+		Launcher launcher = launcherSupplier.get();
+		SummaryGeneratingListener summaryListener = registerListeners(out, reportsDir, launcher);
+
+		LauncherDiscoveryRequest discoveryRequest = new DiscoveryRequestCreator().toDiscoveryRequest(discoveryOptions);
 		launcher.execute(discoveryRequest);
 
 		TestExecutionSummary summary = summaryListener.getSummary();
-		if (summary.getTotalFailureCount() > 0 || options.getDetails() != Details.NONE) {
+		if (summary.getTotalFailureCount() > 0 || outputOptions.getDetails() != Details.NONE) {
 			printSummary(summary, out);
 		}
 
@@ -74,7 +110,7 @@ public class ConsoleTestExecutor {
 	}
 
 	private Optional<ClassLoader> createCustomClassLoader() {
-		List<Path> additionalClasspathEntries = options.getExistingAdditionalClasspathEntries();
+		List<Path> additionalClasspathEntries = discoveryOptions.getExistingAdditionalClasspathEntries();
 		if (!additionalClasspathEntries.isEmpty()) {
 			URL[] urls = additionalClasspathEntries.stream().map(this::toURL).toArray(URL[]::new);
 			ClassLoader parentClassLoader = ClassLoaderUtils.getDefaultClassLoader();
@@ -93,21 +129,21 @@ public class ConsoleTestExecutor {
 		}
 	}
 
-	private SummaryGeneratingListener registerListeners(PrintWriter out, Launcher launcher) {
+	private SummaryGeneratingListener registerListeners(PrintWriter out, Optional<Path> reportsDir, Launcher launcher) {
 		// always register summary generating listener
 		SummaryGeneratingListener summaryListener = new SummaryGeneratingListener();
 		launcher.registerTestExecutionListeners(summaryListener);
 		// optionally, register test plan execution details printing listener
 		createDetailsPrintingListener(out).ifPresent(launcher::registerTestExecutionListeners);
 		// optionally, register XML reports writing listener
-		createXmlWritingListener(out).ifPresent(launcher::registerTestExecutionListeners);
+		createXmlWritingListener(out, reportsDir).ifPresent(launcher::registerTestExecutionListeners);
 		return summaryListener;
 	}
 
-	private Optional<TestExecutionListener> createDetailsPrintingListener(PrintWriter out) {
+	private Optional<DetailsPrintingListener> createDetailsPrintingListener(PrintWriter out) {
 		ColorPalette colorPalette = getColorPalette();
-		Theme theme = options.getTheme();
-		switch (options.getDetails()) {
+		Theme theme = outputOptions.getTheme();
+		switch (outputOptions.getDetails()) {
 			case SUMMARY:
 				// summary listener is always created and registered
 				return Optional.empty();
@@ -117,34 +153,40 @@ public class ConsoleTestExecutor {
 				return Optional.of(new TreePrintingListener(out, colorPalette, theme));
 			case VERBOSE:
 				return Optional.of(new VerboseTreePrintingListener(out, colorPalette, 16, theme));
+			case TESTFEED:
+				return Optional.of(new TestFeedPrintingListener(out, colorPalette));
 			default:
 				return Optional.empty();
 		}
 	}
 
 	private ColorPalette getColorPalette() {
-		if (options.isAnsiColorOutputDisabled()) {
+		if (outputOptions.isAnsiColorOutputDisabled()) {
 			return ColorPalette.NONE;
 		}
-		if (options.getColorPalettePath() != null) {
-			return new ColorPalette(options.getColorPalettePath());
+		if (outputOptions.getColorPalettePath() != null) {
+			return new ColorPalette(outputOptions.getColorPalettePath());
 		}
-		if (options.isSingleColorPalette()) {
+		if (outputOptions.isSingleColorPalette()) {
 			return ColorPalette.SINGLE_COLOR;
 		}
 		return ColorPalette.DEFAULT;
 	}
 
-	private Optional<TestExecutionListener> createXmlWritingListener(PrintWriter out) {
-		return options.getReportsDir().map(reportsDir -> new LegacyXmlReportGeneratingListener(reportsDir, out));
+	private Optional<TestExecutionListener> createXmlWritingListener(PrintWriter out, Optional<Path> reportsDir) {
+		return reportsDir.map(it -> new LegacyXmlReportGeneratingListener(it, out));
 	}
 
 	private void printSummary(TestExecutionSummary summary, PrintWriter out) {
 		// Otherwise the failures have already been printed in detail
-		if (EnumSet.of(Details.NONE, Details.SUMMARY, Details.TREE).contains(options.getDetails())) {
+		if (EnumSet.of(Details.NONE, Details.SUMMARY, Details.TREE).contains(outputOptions.getDetails())) {
 			summary.printFailuresTo(out);
 		}
 		summary.printTo(out);
 	}
 
+	@FunctionalInterface
+	public interface Factory {
+		ConsoleTestExecutor create(TestDiscoveryOptions discoveryOptions, TestConsoleOutputOptions outputOptions);
+	}
 }
