@@ -20,14 +20,13 @@ import java.util.Deque;
 import org.junit.platform.console.options.Theme;
 import org.junit.platform.engine.TestExecutionResult;
 import org.junit.platform.engine.reporting.ReportEntry;
-import org.junit.platform.launcher.TestExecutionListener;
 import org.junit.platform.launcher.TestIdentifier;
 import org.junit.platform.launcher.TestPlan;
 
 /**
  * @since 1.0
  */
-class VerboseTreePrintingListener implements TestExecutionListener {
+class VerboseTreePrintingListener implements DetailsPrintingListener {
 
 	private final PrintWriter out;
 	private final Theme theme;
@@ -60,9 +59,8 @@ class VerboseTreePrintingListener implements TestExecutionListener {
 	public void testPlanExecutionStarted(TestPlan testPlan) {
 		frames.push(System.currentTimeMillis());
 
-		long tests = testPlan.countTestIdentifiers(TestIdentifier::isTest);
-		printf(NONE, "%s", "Test plan execution started. Number of static tests: ");
-		printf(Style.TEST, "%d%n", tests);
+		String prefix = "Test plan execution started. Number of static tests: ";
+		printNumberOfTests(testPlan, prefix);
 		printf(Style.CONTAINER, "%s%n", theme.root());
 	}
 
@@ -70,8 +68,12 @@ class VerboseTreePrintingListener implements TestExecutionListener {
 	public void testPlanExecutionFinished(TestPlan testPlan) {
 		frames.pop();
 
+		printNumberOfTests(testPlan, "Test plan execution finished. Number of all tests: ");
+	}
+
+	private void printNumberOfTests(TestPlan testPlan, String prefix) {
 		long tests = testPlan.countTestIdentifiers(TestIdentifier::isTest);
-		printf(NONE, "%s", "Test plan execution finished. Number of all tests: ");
+		printf(NONE, "%s", prefix);
 		printf(Style.TEST, "%d%n", tests);
 	}
 
@@ -186,4 +188,38 @@ class VerboseTreePrintingListener implements TestExecutionListener {
 		printf(NONE, "%n");
 	}
 
+	@Override
+	public void listTests(TestPlan testPlan) {
+		frames.push(0L);
+		testPlan.accept(new TestPlan.Visitor() {
+			@Override
+			public void preVisitContainer(TestIdentifier testIdentifier) {
+				if (!testPlan.getChildren(testIdentifier).isEmpty()) {
+					printVerticals(theme.entry());
+					printf(Style.CONTAINER, " %s", testIdentifier.getDisplayName());
+					printf(NONE, "%n");
+					frames.push(0L);
+				}
+			}
+
+			@Override
+			public void visit(TestIdentifier testIdentifier) {
+				if (testPlan.getChildren(testIdentifier).isEmpty()) {
+					printVerticals(theme.entry());
+					printf(Style.valueOf(testIdentifier), " %s%n", testIdentifier.getDisplayName());
+					printDetails(testIdentifier);
+				}
+			}
+
+			@Override
+			public void postVisitContainer(TestIdentifier testIdentifier) {
+				if (!testPlan.getChildren(testIdentifier).isEmpty()) {
+					frames.pop();
+					printVerticals(theme.end());
+					printf(Style.CONTAINER, " %s%n", testIdentifier.getDisplayName());
+				}
+			}
+		});
+		frames.pop();
+	}
 }
