@@ -10,7 +10,6 @@
 
 package org.junit.platform.engine;
 
-import static org.apiguardian.api.API.Status.EXPERIMENTAL;
 import static org.apiguardian.api.API.Status.STABLE;
 
 import java.util.List;
@@ -80,7 +79,7 @@ public interface EngineDiscoveryRequest {
 	 * @return the discovery listener; never {@code null}
 	 * @since 1.6
 	 */
-	@API(status = EXPERIMENTAL, since = "1.6")
+	@API(status = STABLE, since = "1.10")
 	default EngineDiscoveryListener getDiscoveryListener() {
 		return EngineDiscoveryListener.NOOP;
 	}
