@@ -10,7 +10,7 @@
 
 package org.junit.platform.engine.support.hierarchical;
 
-import static org.apiguardian.api.API.Status.EXPERIMENTAL;
+import static org.apiguardian.api.API.Status.STABLE;
 
 import java.util.Objects;
 import java.util.concurrent.locks.ReadWriteLock;
@@ -27,7 +27,7 @@ import org.junit.platform.engine.support.hierarchical.Node.ExecutionMode;
  * @since 1.3
  * @see Node#getExecutionMode()
  */
-@API(status = EXPERIMENTAL, since = "1.3")
+@API(status = STABLE, since = "1.10")
 public class ExclusiveResource {
 
 	/**
@@ -44,7 +44,7 @@ public class ExclusiveResource {
 	 *
 	 * @since 1.7
 	 */
-	@API(status = EXPERIMENTAL, since = "1.7")
+	@API(status = STABLE, since = "1.10")
 	public static final String GLOBAL_KEY = "org.junit.platform.engine.support.hierarchical.ExclusiveResource.GLOBAL_KEY";
 
 	static final ExclusiveResource GLOBAL_READ = new ExclusiveResource(GLOBAL_KEY, LockMode.READ);
