@@ -10,6 +10,8 @@
 
 package org.junit.platform.suite.api;
 
+import static org.apiguardian.api.API.Status.STABLE;
+
 import java.lang.annotation.Documented;
 import java.lang.annotation.ElementType;
 import java.lang.annotation.Inherited;
@@ -18,7 +20,6 @@ import java.lang.annotation.RetentionPolicy;
 import java.lang.annotation.Target;
 
 import org.apiguardian.api.API;
-import org.apiguardian.api.API.Status;
 
 /**
  * {@code @SelectUris} specifies the URIs to <em>select</em> when running a test
@@ -33,7 +34,7 @@ import org.apiguardian.api.API.Status;
 @Target(ElementType.TYPE)
 @Inherited
 @Documented
-@API(status = Status.EXPERIMENTAL, since = "1.8")
+@API(status = STABLE, since = "1.10")
 public @interface SelectUris {
 
 	/**
