@@ -1,16 +1,17 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.platform.commons.support;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.platform.commons.support.PreconditionAssertions.assertPreconditionViolationException;
 
 import java.util.List;
 import java.util.function.Function;
@@ -24,7 +25,14 @@ import org.junit.platform.commons.util.ClassUtils;
 class ClassSupportTests {
 
 	@Test
-	void nullSafeToStringDelegates() throws Throwable {
+	void nullSafeToStringPreconditions() {
+		Function<? super Class<?>, ? extends String> mapper = null;
+		assertPreconditionViolationException("Mapping function",
+			() -> ClassSupport.nullSafeToString(mapper, String.class, List.class));
+	}
+
+	@Test
+	void nullSafeToStringDelegates() {
 		assertEquals(ClassUtils.nullSafeToString(String.class, List.class),
 			ClassSupport.nullSafeToString(String.class, List.class));
 
