@@ -34,7 +34,7 @@ import java.util.stream.Stream;
 
 import org.junit.jupiter.api.Test;
 import org.junit.platform.commons.PreconditionViolationException;
-import org.junit.platform.console.options.CommandLineOptions;
+import org.junit.platform.console.options.TestDiscoveryOptions;
 import org.junit.platform.engine.Filter;
 import org.junit.platform.engine.discovery.ClassNameFilter;
 import org.junit.platform.engine.discovery.ClassSelector;
@@ -54,7 +54,7 @@ import org.junit.platform.launcher.LauncherDiscoveryRequest;
  */
 class DiscoveryRequestCreatorTests {
 
-	private final CommandLineOptions options = new CommandLineOptions();
+	private final TestDiscoveryOptions options = new TestDiscoveryOptions();
 
 	@Test
 	void convertsScanClasspathOptionWithoutExplicitRootDirectories() {
@@ -269,10 +269,10 @@ class DiscoveryRequestCreatorTests {
 		assertThat(methodSelectors).hasSize(2);
 		assertThat(methodSelectors.get(0).getClassName()).isEqualTo("com.acme.Foo");
 		assertThat(methodSelectors.get(0).getMethodName()).isEqualTo("m");
-		assertThat(methodSelectors.get(0).getMethodParameterTypes()).isEqualTo("");
+		assertThat(methodSelectors.get(0).getParameterTypeNames()).isEmpty();
 		assertThat(methodSelectors.get(1).getClassName()).isEqualTo("com.example.Bar");
 		assertThat(methodSelectors.get(1).getMethodName()).isEqualTo("method");
-		assertThat(methodSelectors.get(1).getMethodParameterTypes()).isEqualTo("java.lang.Object");
+		assertThat(methodSelectors.get(1).getParameterTypeNames()).isEqualTo("java.lang.Object");
 	}
 
 	@Test
