@@ -27,20 +27,28 @@ class ClassSelectorTests extends AbstractEqualsAndHashCodeTests {
 
 	@Test
 	void equalsAndHashCode() {
-		var selector1 = new ClassSelector("org.example.TestClass");
-		var selector2 = new ClassSelector("org.example.TestClass");
-		var selector3 = new ClassSelector("org.example.X");
+		var selector1 = new ClassSelector(null, "org.example.TestClass");
+		var selector2 = new ClassSelector(null, "org.example.TestClass");
+		var selector3 = new ClassSelector(null, "org.example.X");
 
 		assertEqualsAndHashCode(selector1, selector2, selector3);
 	}
 
 	@Test
 	void preservesOriginalExceptionWhenTryingToLoadClass() {
-		var selector = new ClassSelector("org.example.TestClass");
+		var selector = new ClassSelector(null, "org.example.TestClass");
 
 		var e = assertThrows(PreconditionViolationException.class, selector::getJavaClass);
 
 		assertThat(e).hasMessage("Could not load class with name: org.example.TestClass").hasCauseInstanceOf(
 			ClassNotFoundException.class);
 	}
+
+	@Test
+	void usesClassClassLoader() {
+		var selector = new ClassSelector(getClass());
+
+		assertThat(selector.getClassLoader()).isNotNull().isSameAs(getClass().getClassLoader());
+	}
+
 }
