@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.platform.engine.discovery;
@@ -22,10 +22,19 @@ import org.junit.platform.AbstractEqualsAndHashCodeTests;
 class ClasspathResourceSelectorTests extends AbstractEqualsAndHashCodeTests {
 
 	@Test
-	void equalsAndHashCode() throws Exception {
-		var selector1 = new ClasspathResourceSelector("/foo/bar.txt");
-		var selector2 = new ClasspathResourceSelector("/foo/bar.txt");
-		var selector3 = new ClasspathResourceSelector("/foo/X.txt");
+	void equalsAndHashCode() {
+		var selector1 = new ClasspathResourceSelector("/foo/bar.txt", null);
+		var selector2 = new ClasspathResourceSelector("/foo/bar.txt", null);
+		var selector3 = new ClasspathResourceSelector("/foo/X.txt", null);
+
+		assertEqualsAndHashCode(selector1, selector2, selector3);
+	}
+
+	@Test
+	void equalsAndHashCodeWithFilePosition() {
+		var selector1 = new ClasspathResourceSelector("/foo/bar.txt", FilePosition.from(1));
+		var selector2 = new ClasspathResourceSelector("/foo/bar.txt", FilePosition.from(1));
+		var selector3 = new ClasspathResourceSelector("/foo/bar.txt", FilePosition.from(2));
 
 		assertEqualsAndHashCode(selector1, selector2, selector3);
 	}
