@@ -1,34 +1,42 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.platform.engine.support.hierarchical;
 
-import static java.util.Collections.singleton;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.jupiter.api.Assertions.assertSame;
 import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.platform.engine.TestExecutionResult.Status.ABORTED;
 import static org.junit.platform.engine.TestExecutionResult.Status.FAILED;
 import static org.junit.platform.engine.TestExecutionResult.Status.SUCCESSFUL;
+import static org.junit.platform.engine.TestExecutionResult.successful;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.eq;
 import static org.mockito.Mockito.doThrow;
 import static org.mockito.Mockito.inOrder;
+import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.never;
 import static org.mockito.Mockito.spy;
 import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
+import java.util.Map;
+import java.util.Set;
+import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.CountDownLatch;
+
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.ExtendWith;
+import org.junit.jupiter.api.function.ThrowingConsumer;
 import org.junit.platform.engine.EngineExecutionListener;
 import org.junit.platform.engine.ExecutionRequest;
 import org.junit.platform.engine.TestDescriptor;
@@ -37,8 +45,8 @@ import org.junit.platform.engine.UniqueId;
 import org.junit.platform.engine.support.descriptor.AbstractTestDescriptor;
 import org.junit.platform.engine.support.hierarchical.ExclusiveResource.LockMode;
 import org.junit.platform.engine.support.hierarchical.Node.DynamicTestExecutor;
+import org.junit.platform.launcher.core.ConfigurationParametersFactoryForTests;
 import org.mockito.ArgumentCaptor;
-import org.mockito.InOrder;
 import org.mockito.Mock;
 import org.mockito.Spy;
 import org.mockito.junit.jupiter.MockitoExtension;
@@ -64,18 +72,24 @@ class HierarchicalTestExecutorTests {
 
 	@BeforeEach
 	void init() {
-		ExecutionRequest request = new ExecutionRequest(root, listener, null);
-		executor = new MyExecutor(request, rootContext);
+		executor = createExecutor(new SameThreadHierarchicalTestExecutorService());
+	}
+
+	private HierarchicalTestExecutor<MyEngineExecutionContext> createExecutor(
+			HierarchicalTestExecutorService executorService) {
+		var request = new ExecutionRequest(root, listener, null);
+		return new HierarchicalTestExecutor<>(request, rootContext, executorService,
+			OpenTest4JAwareThrowableCollector::new);
 	}
 
 	@Test
 	void emptyRootDescriptor() throws Exception {
 
-		InOrder inOrder = inOrder(listener, root);
+		var inOrder = inOrder(listener, root);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(root).prepare(rootContext);
 		inOrder.verify(root).shouldBeSkipped(rootContext);
 		inOrder.verify(listener).executionStarted(root);
@@ -89,14 +103,14 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void rootDescriptorWithOneChildContainer() throws Exception {
 
-		MyContainer child = spy(new MyContainer(UniqueId.root("container", "child container")));
+		var child = spy(new MyContainer(UniqueId.root("container", "child container")));
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(listener).executionStarted(root);
 		inOrder.verify(child).prepare(rootContext);
 		inOrder.verify(child).shouldBeSkipped(rootContext);
@@ -112,14 +126,14 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void rootDescriptorWithOneChildLeaf() throws Exception {
 
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "child leaf")));
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "child leaf")));
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(listener).executionStarted(root);
 		inOrder.verify(child).prepare(rootContext);
 		inOrder.verify(child).shouldBeSkipped(rootContext);
@@ -134,11 +148,11 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void skippingAContainer() throws Exception {
 
-		MyContainer child = spy(new MyContainer(UniqueId.root("container", "child container")));
+		var child = spy(new MyContainer(UniqueId.root("container", "child container")));
 		when(child.shouldBeSkipped(rootContext)).thenReturn(Node.SkipResult.skip("in test"));
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
@@ -156,11 +170,11 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void skippingALeaf() throws Exception {
 
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "child leaf")));
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "child leaf")));
 		when(child.shouldBeSkipped(rootContext)).thenReturn(Node.SkipResult.skip("in test"));
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
@@ -178,16 +192,16 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void exceptionInShouldBeSkipped() throws Exception {
 
-		MyContainer child = spy(new MyContainer(UniqueId.root("container", "child container")));
-		RuntimeException anException = new RuntimeException("in skip");
+		var child = spy(new MyContainer(UniqueId.root("container", "child container")));
+		var anException = new RuntimeException("in skip");
 		when(child.shouldBeSkipped(rootContext)).thenThrow(anException);
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, child);
+		var inOrder = inOrder(listener, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(listener).executionStarted(root);
 		inOrder.verify(child).prepare(rootContext);
 		inOrder.verify(child).shouldBeSkipped(rootContext);
@@ -205,16 +219,16 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void exceptionInContainerBeforeAll() throws Exception {
 
-		MyContainer child = spy(new MyContainer(UniqueId.root("container", "child container")));
+		var child = spy(new MyContainer(UniqueId.root("container", "child container")));
 		root.addChild(child);
-		RuntimeException anException = new RuntimeException("in test");
+		var anException = new RuntimeException("in test");
 		when(root.before(rootContext)).thenThrow(anException);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(root).prepare(rootContext);
 		inOrder.verify(root).shouldBeSkipped(rootContext);
 		inOrder.verify(listener).executionStarted(root);
@@ -231,17 +245,17 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void exceptionInContainerAfterAllAndCleanUp() throws Exception {
 
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "child container")));
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "child container")));
 		root.addChild(child);
-		RuntimeException afterException = new RuntimeException("in after()");
+		var afterException = new RuntimeException("in after()");
 		doThrow(afterException).when(root).after(rootContext);
-		RuntimeException cleanUpException = new RuntimeException("in cleanUp()");
+		var cleanUpException = new RuntimeException("in cleanUp()");
 		doThrow(cleanUpException).when(root).cleanUp(rootContext);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
-		InOrder inOrder = inOrder(listener, root, child);
+		var rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var inOrder = inOrder(listener, root, child);
 		inOrder.verify(root).prepare(rootContext);
 		inOrder.verify(root).shouldBeSkipped(rootContext);
 		inOrder.verify(listener).executionStarted(root);
@@ -261,13 +275,13 @@ class HierarchicalTestExecutorTests {
 
 	@Test
 	void exceptionInPrepare() throws Exception {
-		RuntimeException prepareException = new RuntimeException("in prepare()");
+		var prepareException = new RuntimeException("in prepare()");
 		doThrow(prepareException).when(root).prepare(rootContext);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
-		InOrder inOrder = inOrder(listener, root);
+		var rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var inOrder = inOrder(listener, root);
 		inOrder.verify(root).prepare(rootContext);
 		inOrder.verify(listener).executionStarted(root);
 		inOrder.verify(listener).executionFinished(eq(root), rootExecutionResult.capture());
@@ -280,13 +294,13 @@ class HierarchicalTestExecutorTests {
 
 	@Test
 	void exceptionInCleanUp() throws Exception {
-		RuntimeException cleanUpException = new RuntimeException("in cleanUp()");
+		var cleanUpException = new RuntimeException("in cleanUp()");
 		doThrow(cleanUpException).when(root).cleanUp(rootContext);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
-		InOrder inOrder = inOrder(listener, root);
+		var rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var inOrder = inOrder(listener, root);
 		inOrder.verify(root).prepare(rootContext);
 		inOrder.verify(root).shouldBeSkipped(rootContext);
 		inOrder.verify(listener).executionStarted(root);
@@ -303,15 +317,15 @@ class HierarchicalTestExecutorTests {
 
 	@Test
 	void exceptionInShouldBeSkippedAndCleanUp() throws Exception {
-		RuntimeException shouldBeSkippedException = new RuntimeException("in prepare()");
+		var shouldBeSkippedException = new RuntimeException("in prepare()");
 		doThrow(shouldBeSkippedException).when(root).shouldBeSkipped(rootContext);
-		RuntimeException cleanUpException = new RuntimeException("in cleanUp()");
+		var cleanUpException = new RuntimeException("in cleanUp()");
 		doThrow(cleanUpException).when(root).cleanUp(rootContext);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
-		InOrder inOrder = inOrder(listener, root);
+		var rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var inOrder = inOrder(listener, root);
 		inOrder.verify(root).prepare(rootContext);
 		inOrder.verify(root).shouldBeSkipped(rootContext);
 		inOrder.verify(root).cleanUp(rootContext);
@@ -327,16 +341,16 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void exceptionInLeafExecute() throws Exception {
 
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
-		RuntimeException anException = new RuntimeException("in test");
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
+		var anException = new RuntimeException("in test");
 		when(child.execute(eq(rootContext), any())).thenThrow(anException);
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(listener).executionStarted(root);
 		inOrder.verify(root).before(rootContext);
 		inOrder.verify(listener).executionStarted(child);
@@ -350,18 +364,18 @@ class HierarchicalTestExecutorTests {
 	}
 
 	@Test
-	void abortInContainerBeforeAll() throws Exception {
+	void abortInRootBeforeAll() throws Exception {
 
-		MyContainer child = spy(new MyContainer(UniqueId.root("container", "child container")));
+		var child = spy(new MyContainer(UniqueId.root("container", "child container")));
 		root.addChild(child);
-		TestAbortedException anAbortedException = new TestAbortedException("in BeforeAll");
+		var anAbortedException = new TestAbortedException("in BeforeAll");
 		when(root.before(rootContext)).thenThrow(anAbortedException);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var rootExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(root).prepare(rootContext);
 		inOrder.verify(root).shouldBeSkipped(rootContext);
 		inOrder.verify(listener).executionStarted(root);
@@ -376,18 +390,48 @@ class HierarchicalTestExecutorTests {
 	}
 
 	@Test
-	void abortInLeafExecute() throws Exception {
+	void abortInChildContainerBeforeAll() throws Exception {
 
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
-		TestAbortedException anAbortedException = new TestAbortedException("in test");
-		when(child.execute(eq(rootContext), any())).thenThrow(anAbortedException);
+		var child = spy(new MyContainer(UniqueId.root("container", "child container")));
 		root.addChild(child);
+		var anAbortedException = new TestAbortedException("in BeforeAll");
+		when(child.before(rootContext)).thenThrow(anAbortedException);
 
-		InOrder inOrder = inOrder(listener, root, child);
+		var inOrder = inOrder(listener, root, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+
+		inOrder.verify(root).prepare(rootContext);
+		inOrder.verify(root).shouldBeSkipped(rootContext);
+		inOrder.verify(listener).executionStarted(root);
+		inOrder.verify(root).before(rootContext);
+		inOrder.verify(child).shouldBeSkipped(rootContext);
+		inOrder.verify(child).before(rootContext);
+		inOrder.verify(child).after(rootContext);
+		inOrder.verify(listener).executionFinished(eq(child), childExecutionResult.capture());
+		inOrder.verify(root).after(rootContext);
+
+		assertThat(childExecutionResult.getValue().getStatus()).isEqualTo(ABORTED);
+		assertThat(childExecutionResult.getValue().getThrowable()).containsSame(anAbortedException);
+
+		verify(child, never()).execute(any(), any());
+	}
+
+	@Test
+	void abortInLeafExecute() throws Exception {
+
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
+		var anAbortedException = new TestAbortedException("in test");
+		when(child.execute(eq(rootContext), any())).thenThrow(anAbortedException);
+		root.addChild(child);
+
+		var inOrder = inOrder(listener, root, child);
+
+		executor.execute();
+
+		var childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(listener).executionStarted(root);
 		inOrder.verify(root).before(rootContext);
 		inOrder.verify(listener).executionStarted(child);
@@ -403,22 +447,18 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void executesDynamicTestDescriptors() throws Exception {
 
-		UniqueId leafUniqueId = UniqueId.root("leaf", "child leaf");
-		MyLeaf child = spy(new MyLeaf(leafUniqueId));
-		MyLeaf dynamicTestDescriptor = spy(new MyLeaf(leafUniqueId.append("dynamic", "child")));
+		var leafUniqueId = UniqueId.root("leaf", "child leaf");
+		var child = spy(new MyLeaf(leafUniqueId));
+		var dynamicTestDescriptor = spy(new MyLeaf(leafUniqueId.append("dynamic", "child")));
 
-		when(child.execute(any(), any())).thenAnswer(invocation -> {
-			DynamicTestExecutor dynamicTestExecutor = invocation.getArgument(1);
-			dynamicTestExecutor.execute(dynamicTestDescriptor);
-			return invocation.getArgument(0);
-		});
+		when(child.execute(any(), any())).thenAnswer(execute(dynamicTestDescriptor));
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, root, child, dynamicTestDescriptor);
+		var inOrder = inOrder(listener, root, child, dynamicTestDescriptor);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(listener).executionStarted(root);
 		inOrder.verify(child).prepare(rootContext);
 		inOrder.verify(child).shouldBeSkipped(rootContext);
@@ -440,24 +480,23 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void executesDynamicTestDescriptorsUsingContainerAndTestType() throws Exception {
 
-		MyContainerAndTestTestCase child = spy(
-			new MyContainerAndTestTestCase(root.getUniqueId().append("c&t", "child")));
-		MyContainerAndTestTestCase dynamicContainerAndTest = spy(
+		var child = spy(new MyContainerAndTestTestCase(root.getUniqueId().append("c&t", "child")));
+		var dynamicContainerAndTest = spy(
 			new MyContainerAndTestTestCase(child.getUniqueId().append("c&t", "dynamicContainerAndTest")));
-		MyLeaf dynamicLeaf = spy(new MyLeaf(dynamicContainerAndTest.getUniqueId().append("test", "dynamicLeaf")));
+		var dynamicLeaf = spy(new MyLeaf(dynamicContainerAndTest.getUniqueId().append("test", "dynamicLeaf")));
 
 		root.addChild(child);
-		when(child.execute(any(), any())).thenAnswer(registerAndExecute(dynamicContainerAndTest));
-		when(dynamicContainerAndTest.execute(any(), any())).thenAnswer(registerAndExecute(dynamicLeaf));
+		when(child.execute(any(), any())).thenAnswer(execute(dynamicContainerAndTest));
+		when(dynamicContainerAndTest.execute(any(), any())).thenAnswer(execute(dynamicLeaf));
 		when(dynamicLeaf.execute(any(), any())).thenAnswer(invocation -> {
 			throw new AssertionError("test fails");
 		});
 
-		InOrder inOrder = inOrder(listener, root, child, dynamicContainerAndTest, dynamicLeaf);
+		var inOrder = inOrder(listener, root, child, dynamicContainerAndTest, dynamicLeaf);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(listener).executionStarted(root);
 
 		inOrder.verify(child).prepare(rootContext);
@@ -486,21 +525,89 @@ class HierarchicalTestExecutorTests {
 			FAILED, SUCCESSFUL, SUCCESSFUL);
 	}
 
-	private Answer<Object> registerAndExecute(TestDescriptor dynamicChild) {
+	@Test
+	void executesDynamicTestDescriptorsWithCustomListener() {
+
+		var leafUniqueId = UniqueId.root("leaf", "child leaf");
+		var child = spy(new MyLeaf(leafUniqueId));
+		var dynamicTestDescriptor = spy(new MyLeaf(leafUniqueId.append("dynamic", "child")));
+		root.addChild(child);
+
+		var anotherListener = mock(EngineExecutionListener.class);
+		when(child.execute(any(), any())).thenAnswer(
+			useDynamicTestExecutor(executor -> executor.execute(dynamicTestDescriptor, anotherListener)));
+
+		executor.execute();
+
+		var inOrder = inOrder(listener, anotherListener, root, child, dynamicTestDescriptor);
+		inOrder.verify(anotherListener).dynamicTestRegistered(dynamicTestDescriptor);
+		inOrder.verify(anotherListener).executionStarted(dynamicTestDescriptor);
+		inOrder.verify(dynamicTestDescriptor).execute(eq(rootContext), any());
+		inOrder.verify(dynamicTestDescriptor).nodeFinished(rootContext, dynamicTestDescriptor, successful());
+		inOrder.verify(anotherListener).executionFinished(dynamicTestDescriptor, successful());
+	}
+
+	@Test
+	void canAbortExecutionOfDynamicChild() throws Exception {
+
+		var leafUniqueId = UniqueId.root("leaf", "child leaf");
+		var child = spy(new MyLeaf(leafUniqueId));
+		var dynamicTestDescriptor = spy(new MyLeaf(leafUniqueId.append("dynamic", "child")));
+		root.addChild(child);
+
+		var startedLatch = new CountDownLatch(1);
+		var interrupted = new CompletableFuture<Boolean>();
+
+		when(child.execute(any(), any())).thenAnswer(useDynamicTestExecutor(executor -> {
+			var future = executor.execute(dynamicTestDescriptor, EngineExecutionListener.NOOP);
+			startedLatch.await();
+			future.cancel(true);
+			executor.awaitFinished();
+		}));
+		when(dynamicTestDescriptor.execute(any(), any())).thenAnswer(invocation -> {
+			startedLatch.countDown();
+			try {
+				new CountDownLatch(1).await(); // block until interrupted
+				interrupted.complete(false);
+				return null;
+			}
+			catch (InterruptedException e) {
+				interrupted.complete(true);
+				throw e;
+			}
+		});
+
+		var parameters = ConfigurationParametersFactoryForTests.create(Map.of(//
+			DefaultParallelExecutionConfigurationStrategy.CONFIG_STRATEGY_PROPERTY_NAME, "fixed", //
+			DefaultParallelExecutionConfigurationStrategy.CONFIG_FIXED_PARALLELISM_PROPERTY_NAME, "2"));
+
+		try (var executorService = new ForkJoinPoolHierarchicalTestExecutorService(parameters)) {
+			createExecutor(executorService).execute().get();
+		}
+
+		verify(listener).executionFinished(child, successful());
+		assertTrue(interrupted.get(), "dynamic node was interrupted");
+	}
+
+	private Answer<Object> execute(TestDescriptor dynamicChild) {
+		return useDynamicTestExecutor(executor -> executor.execute(dynamicChild));
+	}
+
+	private Answer<Object> useDynamicTestExecutor(ThrowingConsumer<DynamicTestExecutor> action) {
 		return invocation -> {
 			DynamicTestExecutor dynamicTestExecutor = invocation.getArgument(1);
-			dynamicTestExecutor.execute(dynamicChild);
+			action.accept(dynamicTestExecutor);
 			return invocation.getArgument(0);
 		};
 	}
 
 	/**
-	 * Verifies support for blacklisted exceptions.
+	 * Verifies support for unrecoverable exceptions.
 	 */
 	@Test
 	void outOfMemoryErrorInShouldBeSkipped() throws Exception {
-		MyContainer child = spy(new MyContainer(UniqueId.root("container", "child container")));
-		OutOfMemoryError outOfMemoryError = new OutOfMemoryError("in skip");
+		var child = spy(new MyContainer(UniqueId.root("container", "child container")));
+		var outOfMemoryError = new OutOfMemoryError("in skip");
 		when(child.shouldBeSkipped(rootContext)).thenThrow(outOfMemoryError);
 		root.addChild(child);
 
@@ -509,12 +616,12 @@ class HierarchicalTestExecutorTests {
 	}
 
 	/**
-	 * Verifies support for blacklisted exceptions.
+	 * Verifies support for unrecoverable exceptions.
 	 */
 	@Test
-	void outOfMemoryErrorInLeafExecution() throws Exception {
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
-		OutOfMemoryError outOfMemoryError = new OutOfMemoryError("in test");
+	void outOfMemoryErrorInLeafExecution() {
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
+		var outOfMemoryError = new OutOfMemoryError("in test");
 		when(child.execute(eq(rootContext), any())).thenThrow(outOfMemoryError);
 		root.addChild(child);
 
@@ -525,18 +632,18 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void exceptionInAfterDoesNotHideEarlierException() throws Exception {
 
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
 		Exception exceptionInExecute = new RuntimeException("execute");
 		Exception exceptionInAfter = new RuntimeException("after");
 		doThrow(exceptionInExecute).when(child).execute(eq(rootContext), any());
 		doThrow(exceptionInAfter).when(child).after(eq(rootContext));
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, child);
+		var inOrder = inOrder(listener, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(child).execute(eq(rootContext), any());
 		inOrder.verify(child).after(eq(rootContext));
 		inOrder.verify(listener).executionFinished(eq(child), childExecutionResult.capture());
@@ -547,28 +654,24 @@ class HierarchicalTestExecutorTests {
 	}
 
 	@Test
-	void dynamicTestDescriptorsMustNotDeclareExclusiveResources() throws Exception {
+	void dynamicTestDescriptorsMustNotDeclareExclusiveResources() {
 
-		UniqueId leafUniqueId = UniqueId.root("leaf", "child leaf");
-		MyLeaf child = spy(new MyLeaf(leafUniqueId));
-		MyLeaf dynamicTestDescriptor = spy(new MyLeaf(leafUniqueId.append("dynamic", "child")));
+		var leafUniqueId = UniqueId.root("leaf", "child leaf");
+		var child = spy(new MyLeaf(leafUniqueId));
+		var dynamicTestDescriptor = spy(new MyLeaf(leafUniqueId.append("dynamic", "child")));
 		when(dynamicTestDescriptor.getExclusiveResources()).thenReturn(
-			singleton(new ExclusiveResource("foo", LockMode.READ)));
+			Set.of(new ExclusiveResource("foo", LockMode.READ)));
 
-		when(child.execute(any(), any())).thenAnswer(invocation -> {
-			DynamicTestExecutor dynamicTestExecutor = invocation.getArgument(1);
-			dynamicTestExecutor.execute(dynamicTestDescriptor);
-			return invocation.getArgument(0);
-		});
+		when(child.execute(any(), any())).thenAnswer(execute(dynamicTestDescriptor));
 		root.addChild(child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var aTestExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		verify(listener).executionStarted(dynamicTestDescriptor);
 		verify(listener).executionFinished(eq(dynamicTestDescriptor), aTestExecutionResult.capture());
 
-		TestExecutionResult executionResult = aTestExecutionResult.getValue();
+		var executionResult = aTestExecutionResult.getValue();
 		assertThat(executionResult.getStatus()).isEqualTo(FAILED);
 		assertThat(executionResult.getThrowable()).isPresent();
 		assertThat(executionResult.getThrowable().get()).hasMessageContaining(
@@ -578,18 +681,18 @@ class HierarchicalTestExecutorTests {
 	@Test
 	void exceptionInAfterIsReportedInsteadOfEarlierTestAbortedException() throws Exception {
 
-		MyLeaf child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
+		var child = spy(new MyLeaf(UniqueId.root("leaf", "leaf")));
 		Exception exceptionInExecute = new TestAbortedException("execute");
 		Exception exceptionInAfter = new RuntimeException("after");
 		doThrow(exceptionInExecute).when(child).execute(eq(rootContext), any());
 		doThrow(exceptionInAfter).when(child).after(eq(rootContext));
 		root.addChild(child);
 
-		InOrder inOrder = inOrder(listener, child);
+		var inOrder = inOrder(listener, child);
 
 		executor.execute();
 
-		ArgumentCaptor<TestExecutionResult> childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
+		var childExecutionResult = ArgumentCaptor.forClass(TestExecutionResult.class);
 		inOrder.verify(child).execute(eq(rootContext), any());
 		inOrder.verify(child).after(eq(rootContext));
 		inOrder.verify(listener).executionFinished(eq(child), childExecutionResult.capture());
@@ -624,7 +727,7 @@ class HierarchicalTestExecutorTests {
 
 		@Override
 		public MyEngineExecutionContext execute(MyEngineExecutionContext context,
-				DynamicTestExecutor dynamicTestExecutor) throws Exception {
+				DynamicTestExecutor dynamicTestExecutor) {
 			return context;
 		}
 
@@ -647,12 +750,4 @@ class HierarchicalTestExecutorTests {
 		}
 	}
 
-	private static class MyExecutor extends HierarchicalTestExecutor<MyEngineExecutionContext> {
-
-		MyExecutor(ExecutionRequest request, MyEngineExecutionContext rootContext) {
-			super(request, rootContext, new SameThreadHierarchicalTestExecutorService(),
-				OpenTest4JAwareThrowableCollector::new);
-		}
-	}
-
 }
