@@ -1,11 +1,11 @@
 /*
- * Copyright 2015-2018 the original author or authors.
+ * Copyright 2015-2023 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
  * accompanies this distribution and is available at
  *
- * http://www.eclipse.org/legal/epl-v20.html
+ * https://www.eclipse.org/legal/epl-v20.html
  */
 
 package org.junit.platform.engine.support.hierarchical;
@@ -25,7 +25,7 @@ class SingleLockTests {
 	@Test
 	@SuppressWarnings("resource")
 	void acquire() throws Exception {
-		ReentrantLock lock = new ReentrantLock();
+		var lock = new ReentrantLock();
 
 		new SingleLock(lock).acquire();
 
@@ -35,7 +35,7 @@ class SingleLockTests {
 	@Test
 	@SuppressWarnings("resource")
 	void release() throws Exception {
-		ReentrantLock lock = new ReentrantLock();
+		var lock = new ReentrantLock();
 
 		new SingleLock(lock).acquire().close();
 
